@@ -1,11 +1,11 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/child.c,v 1.16 2007/10/12 04:51:22 y-asaba Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/child.c,v 1.22 2008/05/08 03:02:04 y-asaba Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2007	PgPool Global Development Group
+ * Copyright (c) 2003-2008	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -49,6 +49,10 @@
 #include <fcntl.h>
 #endif
 
+#ifdef HAVE_CRYPT_H
+#include <crypt.h>
+#endif
+
 #include "pool.h"
 #include "pool_ip.h"
 #include "md5.h"
@@ -208,9 +212,6 @@ void do_child(int unix_fd, int inet_fd)
 			backend_timer_expired = 0;
 		}
 
-		/* disable timeout */
-		pool_disable_timeout();
-
 		/* read the startup packet */
 	retry_startup:
 		sp = read_startup_packet(frontend);
@@ -269,13 +270,13 @@ void do_child(int unix_fd, int inet_fd)
 			if (frontend->database == NULL)
 			{
 				pool_error("do_child: strdup failed: %s\n", strerror(errno));
-				exit(1);
+				child_exit(1);
 			}
 			frontend->username = strdup(sp->user);
 			if (frontend->username == NULL)
 			{
 				pool_error("do_child: strdup failed: %s\n", strerror(errno));
-				exit(1);
+				child_exit(1);
 			}
 			ClientAuthentication(frontend);
 		}
@@ -402,10 +403,6 @@ void do_child(int unix_fd, int inet_fd)
 
 		}
 
-		/* enable query result read timeout if non strict mode */
-		if (pool_config->replication_strict == 0)
-			pool_enable_timeout();
-
 		connected = 1;
 
  		/* show ps status */
@@ -417,8 +414,11 @@ void do_child(int unix_fd, int inet_fd)
 		if (MAJOR(backend) == PROTO_MAJOR_V2)
 			TSTATE(backend) = 'I';
 
-		/* select load balancing node */
-		backend->info->load_balancing_node = select_load_balancing_node();
+		if (pool_config->load_balance_mode)
+		{
+			/* select load balancing node */
+			backend->info->load_balancing_node = select_load_balancing_node();
+		}
 
 		/* query process loop */
 		for (;;)
@@ -1254,7 +1254,7 @@ static RETSIGTYPE close_idle_connection(int sig)
 static RETSIGTYPE authentication_timeout(int sig)
 {
 	pool_log("authentication is timeout");
-	exit(1);
+	child_exit(1);
 }
 
 /*
@@ -1757,7 +1757,7 @@ int select_load_balancing_node(void)
 	int i;
 
 	/* choose a backend in random manner with weight */
-	selected_slot = 0;
+	selected_slot = MASTER_NODE_ID;
 	total_weight = 0.0;
 
 	for (i=0;i<NUM_BACKENDS;i++)
@@ -1771,7 +1771,7 @@ int select_load_balancing_node(void)
 	total_weight = 0.0;
 	for (i=0;i<NUM_BACKENDS;i++)
 	{
-		if (VALID_BACKEND(i))
+		if (VALID_BACKEND(i) && BACKEND_INFO(i).backend_weight > 0.0)
 		{
 			if(r >= total_weight)
 				selected_slot = i;
