@@ -1,11 +1,11 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /home/prj/pg/ipa-pgpool2/cvsroot/ipa-pgpool2/pgpool_src/pool_rewrite_query.c,v 1.28 2006/08/04 04:41:22 y-mori Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/pool_rewrite_query.c,v 1.10 2008/07/04 05:27:30 y-mori Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2006	PgPool Global Development Group
+ * Copyright (c) 2003-2008	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -33,8 +33,11 @@ static int getInsertRule(ListCell *lc,List *list_t ,DistDefInfo *info, int div_k
 static void examInsertStmt(Node *node,POOL_CONNECTION_POOL *backend,RewriteQuery *message);
 static void examSelectStmt(Node *node,POOL_CONNECTION_POOL *backend,RewriteQuery *message);
 static char *delimistr(char *str);
-static int direct_parallel_query(SelectStmt *stmt,POOL_CONNECTION_POOL *backend);
+static int direct_parallel_query(RewriteQuery *message);
 static int check_whereClause(Node *where);
+static void initMessage(RewriteQuery *message);
+static void initdblink(ConInfoTodblink *dblink, POOL_CONNECTION_POOL *backend);
+static void analyze_debug(RewriteQuery *message);
 
 char *pool_error_message(char *message)
 {
@@ -49,28 +52,33 @@ static int getInsertRule(ListCell *lc,List *list_t ,DistDefInfo *info,int div_ke
 {
 	int loop_counter = 0;
 	int node_number = -1;
+	ListCell *cell;
 
-	foreach(lc,list_t)
+	if(list_t->length != 1)
+		return -1;
+
+  cell = list_head(list_t);
+
+	if(!cell && !IsA(cell,List))
+		return 1;
+
+	foreach(lc,lfirst(cell))
 	{
-		Node *n;
-		ResTarget *target;
 		A_Const *constant;
 		Value value;
 		void *obj = NULL;
 
-		n = lfirst(lc);
-		target = (ResTarget *) n;
+		obj = lfirst(lc);
 
-		if(target->val && IsA(target->val, TypeCast))
+		if(obj && IsA(obj, TypeCast))
 		{
-			TypeCast *type = (TypeCast *) target->val;
+			TypeCast *type = (TypeCast *) obj;
 			obj = type->arg;
+
 			if(!obj)
 			{
 				return -1;
 			}
-		} else {
-			obj = target->val;
 		}
 
 		if(obj && !IsA(obj, A_Const))
@@ -116,17 +124,8 @@ static void examInsertStmt(Node *node,POOL_CONNECTION_POOL *backend, RewriteQuer
 	int dist_def_flag = 0;
 	InsertStmt *insert = (InsertStmt *) node;
 
-    message->type = node->type;
+  message->type = node->type;
 
-	/* the source SELECT ? */
-	if (insert->selectStmt)
-	{
-		/* send  error message to frontend */
-		message->r_code = INSERT_SQL_RESTRICTION;
-		message->r_node = -1;
-		message->rewrite_query = pool_error_message("cannot use SelectStmt in InsertStmt");
-		return;
-	}
 
 	/* insert target table */
 	table = insert->relation;
@@ -139,7 +138,31 @@ static void examInsertStmt(Node *node,POOL_CONNECTION_POOL *backend, RewriteQuer
 		return;
 	}
 
-	list_t = (List *) insert->targetList;
+	/* pool_debug("exam_InsertStmt insert table_name %s:",table->relname); */
+
+	info = pool_get_dist_def_info(MASTER_CONNECTION(backend)->sp->database,
+								  table->schemaname,
+								  table->relname);
+
+	if (!info)
+	{
+		/* send  error message to frontend */
+		message->r_code = INSERT_DIST_NO_RULE;
+		return;
+	}
+
+	/* the source SELECT ? */
+	if (insert->selectStmt && ((SelectStmt *)insert->selectStmt)->targetList)
+	{
+		/* send  error message to frontend */
+		message->r_code = INSERT_SQL_RESTRICTION;
+		message->r_node = -1;
+		message->rewrite_query = pool_error_message("cannot use SelectStmt in InsertStmt");
+		return;
+	}
+
+	list_t = (List *)(((SelectStmt *)insert->selectStmt)->valuesLists);
+
 	if (!list_t)
 	{
 		/* send  error message to frontend */
@@ -150,22 +173,20 @@ static void examInsertStmt(Node *node,POOL_CONNECTION_POOL *backend, RewriteQuer
 	}
 	
 	/* number of target list */
-	cell_num = list_t->length;
 
-	/* pool_debug("exam_InsertStmt insert table_name %s:",table->relname); */
-
-	info = pool_get_dist_def_info(MASTER_CONNECTION(backend)->sp->database,
-								  table->schemaname,
-								  table->relname);
-
-	if (!info)
+	if(list_t->length == 1 && IsA(lfirst(list_head(list_t)),List))
 	{
-		/* send  error message to frontend */
-		message->r_code = INSERT_SQL_RESTRICTION;
-		message->r_node = -1;
-		message->rewrite_query = pool_error_message("cannot find dist_def table info");
-		return;
-	}
+		cell_num = ((List *) lfirst(list_head(list_t)))->length;
+	} 
+	else 
+	{
+			/* send  error message to frontend */
+			message->r_code = INSERT_SQL_RESTRICTION;
+			message->r_node = -1;
+			message->rewrite_query = pool_error_message("cannot analzye this InsertStmt");
+			return;
+  }
+
 
 	/* Is the target columns ?*/
 	if (!insert->cols)
@@ -173,7 +194,7 @@ static void examInsertStmt(Node *node,POOL_CONNECTION_POOL *backend, RewriteQuer
 		div_key_num = info->dist_key_col_id;
 		dist_def_flag = 1;
 
-		pool_debug("div key num %d, div_key columname %s",div_key_num,info->col_list[div_key_num]);
+		pool_debug("cell number %d, div key num %d, div_key columname %s",cell_num,div_key_num,info->col_list[div_key_num]);
 
 		if (cell_num < div_key_num)
 		{
@@ -184,7 +205,7 @@ static void examInsertStmt(Node *node,POOL_CONNECTION_POOL *backend, RewriteQuer
 			return;
 		}
 
-    }
+  }
 	else 
 	{
 		List *list_cols = (List *) insert->cols;
@@ -245,132 +266,151 @@ static void examSelectStmt(Node *node,POOL_CONNECTION_POOL *backend,RewriteQuery
 	static ConInfoTodblink dblink;
 
 	/* initialize dblink info */
-	dblink.dbname =  MASTER_CONNECTION(backend)->sp->database;
-	dblink.hostaddr = pool_config->pgpool2_hostname;
-	dblink.user = MASTER_CONNECTION(backend)->sp->user;
-	dblink.port = pool_config->port;
-	dblink.password = MASTER_CONNECTION(backend)->con->password;
+	initdblink(&dblink,backend);
 
 	/* initialize  message */
+	initMessage(message);
 	message->type = node->type;
 	message->r_code = SELECT_DEFAULT;
-	message->dbname = NULL;
-	message->table_relname = NULL;
-	message->table_alias = NULL;
-	message->schemaname = NULL;
-	message->dbname = NULL;
 
 	nodeToRewriteString(message,&dblink,node);
 }
 
+
+static void initMessage(RewriteQuery *message)
+{
+	message->r_code = 0;
+	message->r_node = 0;
+	message->column = 0;
+	message->virtual_num = 0;
+	message->is_pg_catalog = false;
+	message->is_loadbalance = false;
+	message->is_parallel = false;
+	message->table_relname = NULL;
+	message->table_alias = NULL;
+	message->dbname = NULL;
+	message->schemaname = NULL;
+	message->rewrite_query = NULL;
+	message->rewritelock = -1;
+	message->ignore_rewrite = -1;
+	message->ret_num = 0;
+}
+
+static void initdblink(ConInfoTodblink *dblink,POOL_CONNECTION_POOL *backend)
+{
+	dblink->dbname =  MASTER_CONNECTION(backend)->sp->database;
+	dblink->hostaddr = pool_config->pgpool2_hostname;
+	dblink->user = MASTER_CONNECTION(backend)->sp->user;
+	dblink->port = pool_config->port;
+	dblink->password = MASTER_CONNECTION(backend)->con->password;
+}
+
 int IsSelectpgcatalog(Node *node,POOL_CONNECTION_POOL *backend)
 {
 	static ConInfoTodblink dblink;
 	static RewriteQuery message;
 
 	/* initialize dblink info */
-	dblink.dbname =  MASTER_CONNECTION(backend)->sp->database;
-	dblink.hostaddr = pool_config->pgpool2_hostname;
-	dblink.user = MASTER_CONNECTION(backend)->sp->user;
-	dblink.port = pool_config->port;
-	dblink.password = MASTER_CONNECTION(backend)->con->password;
+	initdblink(&dblink,backend);
 
 	/* initialize  message */
+	initMessage(&message);
+
 	message.type = node->type;
-	message.r_code = SELECT_CHECK_PGCATALOG;
-	message.dbname = NULL;
-	message.table_relname = NULL;
-	message.table_alias = NULL;
-	message.schemaname = NULL;
-	message.dbname = NULL;
 
-	nodeToRewriteString(&message,&dblink,node);
+	initdblink(&dblink,backend);
 
-	if(message.r_code == SELECT_PGCATALOG)
+	if(message.is_pg_catalog) 
+	{
+		pool_debug("Isselectpgcatalog %d",message.is_pg_catalog);
 		return 1;
+	}
 	else
+	{
 		return 0;
+	}
 }
 
-RewriteQuery *rewrite_query_stmt(Node *node,POOL_CONNECTION *frontend,POOL_CONNECTION_POOL *backend)
+RewriteQuery *rewrite_query_stmt(Node *node,POOL_CONNECTION *frontend,POOL_CONNECTION_POOL *backend,RewriteQuery *message)
 {
-	static RewriteQuery message;
-
 	switch(node->type)
 	{
 		case T_SelectStmt:
+		{
+			SelectStmt *stmt = (SelectStmt *)node;
+			if(stmt->intoClause)
 			{
-				SelectStmt *stmt = (SelectStmt *)node;
-				if(stmt->into)
-				{
-					pool_send_error_message(frontend, MAJOR(backend), "XX000",
+				pool_send_error_message(frontend, MAJOR(backend), "XX000",
 										"pgpool2 sql restriction",
 										"cannot use select into ...", "", __FILE__,
 										__LINE__);
 
 
-					pool_send_readyforquery(frontend);
-					message.status=POOL_CONTINUE;
-					break;
-				}
-			
-				examSelectStmt(node,backend,&message);
-
-				if(message.r_code != SELECT_PGCATALOG &&
-					message.r_code != SELECT_RELATION_ERROR)
-				{
-					POOL_CONNECTION_POOL_SLOT *system_db = pool_system_db_connection();
-					message.status = OneNode_do_command(frontend, 
-													system_db->con, 
-													message.rewrite_query,
-													backend->info->database);
-				} else {
-					if(TSTATE(backend) == 'T' && SELECT_RELATION_ERROR)
-					{
-						pool_debug("pool_rewrite_stmt(select): inside transaction");
-						message.rewrite_query = nodeToString(node);
-						message.status = pool_parallel_exec(frontend,backend,message.rewrite_query,node,true);
-					} 
-					else { 
-					
-						pool_debug("pool_rewrite_stmt: executed by Master");
-						message.rewrite_query = nodeToString(node);
-						message.status = OneNode_do_command(frontend, 
-													MASTER(backend),
-													message.rewrite_query,
-													backend->info->database);
-					}
-				}
-				pool_debug("pool_rewrite_stmt: XXX message_code %d",message.r_code);
+				pool_send_readyforquery(frontend);
+				message->status=POOL_CONTINUE;
+				break;
 			}
- 			break;
+
+			examSelectStmt(node,backend,message);
+
+			if (message->r_code != SELECT_PGCATALOG &&
+				message->r_code != SELECT_RELATION_ERROR)
+			{
+				POOL_CONNECTION_POOL_SLOT *system_db = pool_system_db_connection();
+				message->status = OneNode_do_command(frontend, 
+													system_db->con, 
+													message->rewrite_query,
+													backend->info->database);
+			}
+			else
+			{
+				if(TSTATE(backend) == 'T' &&
+				   message->r_code == SELECT_RELATION_ERROR)
+				{
+					pool_debug("pool_rewrite_stmt(select): inside transaction");
+					message->rewrite_query = nodeToString(node);
+					message->status = pool_parallel_exec(frontend,backend,message->rewrite_query,node,true);
+				} 
+				else
+				{ 
+					pool_debug("pool_rewrite_stmt: executed by Master");
+					message->rewrite_query = nodeToString(node);
+					message->status = OneNode_do_command(frontend, 
+														MASTER(backend),
+														message->rewrite_query,
+														backend->info->database);
+				}
+			}
+			pool_debug("pool_rewrite_stmt: select message_code %d",message->r_code);
+		}
+		break;
 			
 		case T_InsertStmt:
-			examInsertStmt(node,backend,&message);
+			examInsertStmt(node,backend,message);
 
-			if(message.r_code == 0 )
+			if(message->r_code == 0 )
 			{
-				message.status = OneNode_do_command(frontend, 
-													CONNECTION(backend,message.r_node),
-													message.rewrite_query,
+				message->status = OneNode_do_command(frontend, 
+													CONNECTION(backend,message->r_node),
+													message->rewrite_query,
 													backend->info->database);
 			}
-			else if (message.r_code == INSERT_SQL_RESTRICTION)
+			else if (message->r_code == INSERT_SQL_RESTRICTION)
 			{
 				pool_send_error_message(frontend, MAJOR(backend), "XX000",
 										"pgpool2 sql restriction",
-										message.rewrite_query, "", __FILE__,
+										message->rewrite_query, "", __FILE__,
 										__LINE__);
 
 				if(TSTATE(backend) == 'T')
 				{
 					pool_debug("rewrite_query_stmt(insert): inside transaction");
-					message.status = pool_parallel_exec(frontend,backend, "POOL_RESET_TSTATE",node,false);
+					message->status = pool_parallel_exec(frontend,backend, "POOL_RESET_TSTATE",node,false);
 				}
 				else
 				{
 					pool_send_readyforquery(frontend);
-					message.status=POOL_CONTINUE;
+					message->status=POOL_CONTINUE;
 				}
 			}
 			break;
@@ -380,17 +420,18 @@ RewriteQuery *rewrite_query_stmt(Node *node,POOL_CONNECTION *frontend,POOL_CONNE
 			break;
 #endif
 		default:
-			message.type = node->type;
-			message.status = POOL_CONTINUE;
+			message->type = node->type;
+			message->status = POOL_CONTINUE;
 			break;
 	}
 
-	pool_debug("pool_rewrite_stmt: XXX rule %d",node->type);
+	pool_debug("pool_rewrite_stmt: query rule %d",node->type);
 	
-	return &message;
+	return message;
 }
 
 #define POOL_PARALLEL "pool_parallel"
+#define POOL_LOADBALANCE "pool_loadbalance"
 
 static int check_whereClause(Node *where)
 {
@@ -421,64 +462,12 @@ static int check_whereClause(Node *where)
 	}
 }
 
-static int direct_parallel_query(SelectStmt *stmt,POOL_CONNECTION_POOL *backend)
+static int direct_parallel_query(RewriteQuery *message)
 {
-
-	if(stmt->lockingClause)
-	{
-		pool_debug("lockingClasue is exist");	
+	if(message && message->analyze[0] && message->analyze[0]->state == 'P')
 		return 1;
-	}
-
-	if (!stmt->distinctClause && !stmt->into && !stmt->intoColNames &&
-		!stmt->groupClause && !stmt->havingClause && !stmt->sortClause &&
-		!stmt->limitOffset && !stmt->limitCount &&!stmt->larg && !stmt->rarg)
-	{
-		if(stmt->fromClause && (list_length(stmt->fromClause) == 1)
-			&& list_head(stmt->fromClause) && lfirst(list_head(stmt->fromClause))
-			&& IsA(lfirst(list_head(stmt->fromClause)),RangeVar))
-		{
-			ListCell *lc;
-			DistDefInfo *info = NULL;
-			RangeVar *var = NULL;
-
-			var = (RangeVar *) lfirst(list_head(stmt->fromClause));
-			info = pool_get_dist_def_info (MASTER_CONNECTION(backend)->sp->database, var->schemaname, var->relname);
-
-			if(!info)
-				return 0;
-
-			if(stmt->whereClause && 
-				(check_whereClause(stmt->whereClause)))
-			{
-				return 0;
-			}
-
-			foreach (lc, stmt->targetList)
-       		{
-           		Node *n = lfirst(lc);
-           		if (IsA(n, ResTarget))
-           		{
-					ResTarget *target = (ResTarget *) n;
-					if (target->val && (IsA(target->val, FuncCall) || 
-						IsA(target->val,CaseExpr) || IsA(target->val,SubLink)))
-					{
-						return 0;
-					}
-					else if(target->val && (IsA(target->val, TypeCast)))
-					{
-						TypeCast *type = (TypeCast *)target->val;
-						if(type->arg && (IsA(type->arg,FuncCall) ||
-						   IsA(type->arg,CaseExpr) || IsA(type->arg,SubLink)))
-							return 0;
-					}
-				}
-			}
-
-			return 1;				
-		}
-	}
-	return 0;
+	else
+		return 0;
 }
 
 static char *delimistr(char *str)
@@ -491,10 +480,12 @@ static char *delimistr(char *str)
 
 	for(i = 0; i < len; i++)
 	{
-		unsigned char ch = (unsigned char) str[i];
+		char c = (unsigned char) str[i];
 		if((i != 0) && (i != len -1))
 		{
-			result[j] = (char) ch;
+			if(c=='\'' && (char) str[i+1]=='\'') 
+				i++;
+			result[j] = c;
 			j++;
 		}
 	}
@@ -504,9 +495,25 @@ static char *delimistr(char *str)
 	return result;
 }
 
-char *is_parallel_query(Node *node, POOL_CONNECTION_POOL *backend)
+void analyze_debug(RewriteQuery *message)
 {
-	char *parallel = NULL;
+	int analyze_num,i;
+	analyze_num = message->analyze_num;
+
+	for(i = 0; i< analyze_num; i++)
+	{
+		AnalyzeSelect *analyze = message->analyze[i];
+		pool_debug("analyze_debug :select no(%d), last select(%d), last_part(%d), state(%c)",
+             analyze->now_select,analyze->last_select,analyze->call_part,analyze->state);
+	}	
+}
+
+RewriteQuery *is_parallel_query(Node *node, POOL_CONNECTION_POOL *backend)
+{
+	static RewriteQuery message;
+	static ConInfoTodblink dblink;
+
+	initMessage(&message);
 
 	if (IsA(node, SelectStmt))
 	{
@@ -516,48 +523,101 @@ char *is_parallel_query(Node *node, POOL_CONNECTION_POOL *backend)
 
 		stmt = (SelectStmt *) node;
 
-		direct_ok = direct_parallel_query(stmt,backend);
-		if(direct_ok == 1)
-		{
-			parallel = nodeToString(node);
-			pool_debug("can pool_parallel_exec %s",parallel);	
-			return parallel;
-		}
-
-		if (stmt->distinctClause || stmt->into || stmt->intoColNames ||
+		/* LOADBALANCE OR PARALLEL */
+		if (!(stmt->distinctClause || stmt->intoClause ||
 			stmt->fromClause || stmt->groupClause || stmt->havingClause ||
 			stmt->sortClause || stmt->limitOffset || stmt->limitCount ||
-			stmt->lockingClause || stmt->larg || stmt->rarg)
-			return NULL;
-
-		n = lfirst(list_head(stmt->targetList));
-		if (IsA(n, ResTarget))
+			stmt->lockingClause || stmt->larg || stmt->rarg) && 
+			(n = lfirst(list_head(stmt->targetList))) && IsA(n, ResTarget))
 		{
 			ResTarget *target = (ResTarget *) n;
 		
 			if (target->val && IsA(target->val, FuncCall))
 			{
 				FuncCall *func = (FuncCall *) target->val;
-				if (list_length(func->funcname) == 1 &&
-					strcmp(strVal(lfirst(list_head(func->funcname))),
-						   POOL_PARALLEL) == 0)
+				if (list_length(func->funcname) == 1 && func->args)
 				{
 					Node *func_args = (Node *) lfirst(list_head(func->args));
-					parallel = delimistr(nodeToString(func_args));
-					return parallel;
-				}
+					message.rewrite_query = delimistr(nodeToString(func_args));
+
+					/* PARALLEL */
+					if(strcmp(strVal(lfirst(list_head(func->funcname))),
+						   POOL_PARALLEL) == 0)
+					{
+						message.r_code = SEND_PARALLEL_ENGINE;
+						message.is_parallel = true;
+						message.is_loadbalance = false;
+						pool_debug("can pool_parallel_exec %s",message.rewrite_query);	
+						return &message;
+					} 
+					else /* LOADBALANCE */ 
+					if(strcmp(strVal(lfirst(list_head(func->funcname))),
+						   						POOL_LOADBALANCE) == 0)
+					{
+						message.r_code = SEND_LOADBALANCE_ENGINE;
+						message.is_loadbalance = true;
+						message.is_parallel = false;
+						pool_debug("can loadbalance_mode %s",message.rewrite_query);	
+						return &message;
+					}
+				} 
 			}
 		}
+		
+    /* ANALYZE QUERY */
+		message.r_code = SELECT_ANALYZE;
+		message.is_loadbalance = true;
+
+		initdblink(&dblink,backend);
+		nodeToRewriteString(&message,&dblink,node);
+
+		if(message.is_pg_catalog)
+		{
+			message.is_loadbalance = false;
+			message.is_parallel = false;
+			pool_debug("is_parallel_query: query is done by loadbalance(pgcatalog)");
+			return &message;
+		}
+
+		if(message.is_loadbalance)
+		{
+			message.is_parallel = false;
+			pool_debug("is_parallel_query: query is done by loadbalance");
+			return &message;
+		}
+
+		analyze_debug(&message);
+
+		/* PARALLEL*/
+		direct_ok = direct_parallel_query(&message);
+		if(direct_ok == 1)
+		{
+			message.rewrite_query = nodeToString(node);
+			message.is_parallel = true;
+			message.is_loadbalance = false;
+			pool_debug("can pool_parallel_exec %s",message.rewrite_query);	
+			return &message;
+		}
 	}
 	else if (IsA(node, CopyStmt))
 	{
 		CopyStmt *stmt = (CopyStmt *)node;
+
 		if (stmt->is_from == FALSE && stmt->filename == NULL)
 		{
-			parallel = nodeToString(stmt);
-			return parallel;
+			RangeVar *relation = (RangeVar *)stmt->relation;
+
+			/* check on distribution table or replicate table */
+
+			if(pool_get_dist_def_info (MASTER_CONNECTION(backend)->sp->database, relation->schemaname, relation->relname))
+			{
+				message.rewrite_query = nodeToString(stmt);
+				message.is_parallel = true;
+				message.is_loadbalance = false;
+				message.r_code = SEND_PARALLEL_ENGINE;
+			}
 		}
 	}
 
-	return NULL;
+	return &message;
 }
